/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * include/blalloc.h: block allocator header file
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: blalloc.h,v 1.14 2004/04/10 14:12:23 rageircd Exp $
 */

#ifndef __blalloc_include__
#define __blalloc_include__

#include "setup.h"
#include "dlink.h"
#include <stddef.h>

typedef struct _block_heap BlockHeap;

#ifdef USE_BLALLOC

typedef struct _block Block;

struct _block_heap {
	size_t elemSize;
	int elemsPerBlock;
	int mapSize;
	int freeElems;
	dlink_node node;
	dlink_list block_list;
};

struct _block {
	void *elems;
	void *endElem;
	int freeElems;
	unsigned long *allocMap;
	dlink_node self;
};

extern void init_blockheap(int);
extern void blockheap_cleanup();

extern BlockHeap *BlockHeapCreate(size_t, int);
extern void BlockHeapDestroy(BlockHeap *);
extern void BlockHeapGarbageCollect(BlockHeap *);
extern void *BlockHeapALLOC(BlockHeap *);
extern void BlockHeapFREE(BlockHeap *, void *);
extern void BlockHeapUsage(BlockHeap *, int *, unsigned long *);

extern dlink_list heap_list;

#else

#define BlockHeapCreate(es, epb) ((BlockHeap *)es)
#define BlockHeapGarbageCollect(x)
#define BlockHeapUsage(xx,yy,zz)

#endif

extern BlockHeap *client_heap;
extern BlockHeap *localclient_heap;
extern BlockHeap *user_heap;
extern BlockHeap *localuser_heap;
extern BlockHeap *channel_heap;
extern BlockHeap *chanmember_heap;
extern BlockHeap *channelban_heap;
extern BlockHeap *slink_heap;
extern BlockHeap *dlinknode_heap;
#ifdef FLUD
extern BlockHeap *fludbot_heap;
#endif
#ifdef USE_THROTTLE
extern BlockHeap *throttle_heap;
extern BlockHeap *hashent_heap;
#endif
extern BlockHeap *connauth_heap;
extern BlockHeap *dnsquery_heap;
extern BlockHeap *ipentry_heap;

#endif /* __blalloc_include__ */
