/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * include/common.h: common definition files
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: common.h,v 1.25 2004/04/08 21:24:27 rageircd Exp $
 */

#ifndef	__common_include__
#define __common_include__

#include "setup.h"

#ifndef NULL
#define NULL 0
#endif

#ifndef FALSE
#define FALSE 0
#endif

#ifndef TRUE
#define TRUE 1
#endif

#define SHOW_IP 1
#define HIDE_IP 2

#ifndef STDC_HEADERS
extern void *malloc(), *calloc(), free();
#else
#ifdef MALLOCH
#include MALLOCH
#endif
#endif

#define IRCD_MIN(a, b)		((a) < (b) ? (a) : (b))
#define IRCD_MAX(a, b)		((a) > (b) ? (a) : (b))

#define SENDQLENGTH(x)		((x)->localClient->class != NULL && ((x)->localClient->class->sendq_length > 0) ? \
					(x)->localClient->class->sendq_length : MAXSENDQLENGTH)
#define PINGTIME(x)		((x)->localClient->class != NULL && ((x)->localClient->class->ping_time > 0) ? \
					(x)->localClient->class->ping_time : DEFAULT_PINGTIME)

#ifdef USE_ASSERT
extern void fatal(char *, int, char *);
#define _ASSERT(xx, ff, ll)	if (!(xx)) fatal(ff, ll, #xx)
#define ASSERT(xx)		_ASSERT(xx, __FILE__, __LINE__)
#else
#define ASSERT(xx)
#endif

/* VA_COPY - namely for x86_64 and PowerPC architectures.
 * Copyright Rossi 'vejeta' Marcello (vjt@users.sourceforge.net)
 */
#ifdef __powerpc__
#ifdef __NetBSD__
#define VA_COPY va_copy
#elif defined(__FreeBSD__) || defined(__linux__)
#define VA_COPY __va_copy
#endif
#elif defined(__x86_64)
#define VA_COPY __va_copy
#else
#define VA_COPY(x, y) x = y
#endif

extern unsigned char tolowertab[];
extern unsigned char touppertab[];
extern unsigned char char_atribs[];

#undef tolower
#undef toupper
#undef isalpha
#undef isdigit
#undef isxdigit
#undef isalnum
#undef isprint
#undef isascii
#undef isgraph
#undef ispunct
#undef islower
#undef isupper
#undef isspace
#undef iscntrl

#define tolower(c)	(tolowertab[(u_char)(c)])
#define toupper(c)	(touppertab[(u_char)(c)])
#define PRINT		1
#define CNTRL		2
#define ALPHA		4
#define PUNCT		8
#define DIGIT		16
#define SPACE		32
#define	iscntrl(c)	(char_atribs[(u_char)(c)]&CNTRL)
#define isalpha(c)	(char_atribs[(u_char)(c)]&ALPHA)
#define isspace(c)	(char_atribs[(u_char)(c)]&SPACE)
#define islower(c)	((char_atribs[(u_char)(c)]&ALPHA) && ((u_char)(c) > 0x5f))
#define isupper(c)	((char_atribs[(u_char)(c)]&ALPHA) && ((u_char)(c) < 0x60))
#define isdigit(c)	(char_atribs[(u_char)(c)]&DIGIT)
#define	isxdigit(c)	(isdigit(c) || ('a' <= (c) && (c) <= 'f') || ('A' <= (c) && (c) <= 'F'))
#define isalnum(c)	(char_atribs[(u_char)(c)]&(DIGIT|ALPHA))
#define isprint(c)	(char_atribs[(u_char)(c)]&PRINT)
#define isascii(c)	(((u_char)(c) >= 0) && ((u_char)(c) <= 0x7f))
#define isgraph(c)	((char_atribs[(u_char)(c)]&PRINT) && ((u_char)(c) != 0x32))
#define ispunct(c)	(!(char_atribs[(u_char)(c)]&(CNTRL|ALPHA|DIGIT)))

#endif /* __common_include__ */
