/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * include/hook.h: hooks header file
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: hook.h,v 1.25 2004/05/13 16:12:53 rageircd Exp $
 */

#ifndef	__hook_include__
#define	__hook_include__

#include "setup.h"

typedef struct _hook Hook;
typedef struct _hook_event HookEvent;
typedef struct _hook_data HookData;

struct _hook {
	char *name;
	dlink_list events;
	Module *owner;
};

struct _hook_event {
	Hook *hook;
	int (*event)();
	Module *owner;
};

struct _hook_data {
	aClient *cptr;
	aClient *sptr;
	aClient *acptr;
	aChannel *chptr;

	void *v;
	int i;
	time_t t;
	char *c;
};

extern dlink_list hook_list;

extern Hook *h_pre_access;
extern Hook *h_post_access;
extern Hook *h_post_motd;
extern Hook *h_conf_parse;
extern Hook *h_conf_test;
extern Hook *h_conf_verify;
extern Hook *h_conf_complete;
extern Hook *h_conf_rehash;
extern Hook *h_pre_netio;
extern Hook *h_post_netio;
extern Hook *h_exit_user_local;
extern Hook *h_exit_user_remote;
extern Hook *h_exit_server;
extern Hook *h_exit_unknown;
extern Hook *h_pre_connauth;
extern Hook *h_post_register_user;
extern Hook *h_post_oper;

extern void hook_init();
extern Hook *hook_find_from_module(char *, char *);
extern Hook *hook_find(char *);
extern Hook *hook_add(char *);
extern void hook_del(Hook *);
extern HookEvent *hook_add_event(Hook *, int (*)());
extern void hook_del_event(HookEvent *);
extern void hook_del_events(Hook *);
extern void hook_report(aClient *);

static inline void hook_run(Hook *hook, HookData *data)
{
	dlink_node *node;
	int (*event)();

	ASSERT(hook != NULL);

	DLINK_FOREACH(hook->events.head, node) {
		event = ((HookEvent *)node->data)->event;
		(*event)(data);
	}
}

static inline int hook_run_until(Hook *hook, HookData *data, int until)
{
	dlink_node *node;
	int (*event)(), ret = 0;

	ASSERT(hook != NULL);

	DLINK_FOREACH(hook->events.head, node) {
		event = ((HookEvent *)node->data)->event;
		if ((ret = (*event)(data)) == until) {
			break;
		}
	}

	return ret;
}

static inline int hook_run_untilnot(Hook *hook, HookData *data, int untilnot)
{
	dlink_node *node;
	int (*event)(), ret = 0;

	ASSERT(hook != NULL);

	DLINK_FOREACH(hook->events.head, node) {
		event = ((HookEvent *)node->data)->event;
		if ((ret = (*event)(data)) != untilnot) {
			break;
		}
	}

	return ret;
}

#endif /* __hook_include__ */
