/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * include/struct.h: structures header file
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: struct.h,v 1.236 2004/05/13 16:12:53 rageircd Exp $
 */

#ifndef	__struct_include__
#define __struct_include__

#include "config.h"
#ifndef CONFIG_H_LEVEL_18
#error Incorrect config.h for this revision of ircd.
#endif

#include "queue.h"
#include "setup.h"
#include <stdio.h>
#include <sys/types.h>
#include <time.h>
#include <netinet/in.h>
#include <netdb.h>
#ifdef HAVE_STDDEF_H
#include <stddef.h>
#endif
#include "hash.h"
#include "../adns/adns.h"

typedef struct Client aClient;
typedef struct Client_local LocalClient;
typedef struct User anUser;
typedef struct User_local LocalUser;
typedef struct Server aServer;
typedef struct Channel aChannel;
typedef struct slink SLink;
typedef struct channelban channelBan;
typedef struct channelMember chanMember;
typedef struct SMode Mode;
typedef struct Watch aWatch;
typedef struct ListOptions LOpts;
typedef struct Whowas aWhowas;
typedef struct iphash_entry IPEntry;
typedef struct module Module;
typedef struct event Event;
typedef struct scache_entry ServerCache;
#ifdef USE_THROTTLE
typedef struct hashent_t hashent;
typedef struct hashent_list_t hashent_list;
typedef struct hash_table_t hash_table;
typedef struct throttle_t throttle;
#endif
typedef struct listener Listener;
typedef struct dns_query DNSQuery;
typedef struct connauth ConnectAuth;
typedef struct message_file MessageFile;
typedef struct suid SUID;
typedef struct capab_t Capability;
typedef struct ircd_t IRCd;
typedef struct auth Auth;

#include "conf2.h"
#include "sbuf.h"
#include "ssl.h"
#include "dlink.h"

#define	HOSTLEN			63	/* Length of hostname.  Updated to comply with RFC1123 */
#define HOSTIPLEN		15	/* Length of dotted quad form of IPv4 */
#define	NICKLEN			30	/* Length of nickname */
#define MAX_DATE_STRING		32	/* Length of date string */
#define	USERLEN			10	/* Length of username */
#define	REALLEN			50	/* Length of realname */
#define	TOPICLEN		307	/* Length of topic */
#define	KILLLEN			400	/* Length of kill */
#define	CHANNELLEN		32	/* Length of channel name */
#define	PASSWDLEN		63	/* Length of passwords */
#define	KEYLEN			23	/* Length of channel key */
#define	BUFSIZE			512	/* Length of internal buffers */
#define	MAXRECIPIENTS		20
#define	MAXBANS			45
#define	MAXSILES		5
#define MAXSILELENGTH		128
#define MAXTARGETS		10
#define MAXDCCALLOW		5
#define MAXACCEPT		20
#define MAXWHOREPLIES		200
#define MAXWATCH		128
#define MAXKILLS		20

#define LUSERS_CACHE_TIME	180
#define SCACHE_HASH_SIZE	257
#define IP_HASH_SIZE		0x1000
#define	USERHOST_REPLYLEN	(NICKLEN + HOSTLEN + USERLEN + 5)
#define ID_MAP_SIZE		4096
#define MAX_CLIENT_RECVQ	8192

#define MAX_SERVID              0xFFF	/* max serv id: 4,096 */
#define MIN_SERVID              0x1	/* min serv id: 1 */

#define MAX_USERID              0xFFFFE /* max user id: 1,048,573 */
#define MIN_USERID              0x1	/* min user id: 1 */

#ifndef offsetof
#define	offsetof(t, m)		(int)((&((t *)0L)->m))
#endif
#define timeofday		(Internal.ircd_time.tv_sec)

#define	STAT_CONNECTING		-4
#define	STAT_HANDSHAKE		-3
#define	STAT_ME			-2
#define	STAT_UNKNOWN		-1
#define	STAT_SERVER		0
#define	STAT_CLIENT		1

#define	IsRegisteredUser(x)	((x)->status == STAT_CLIENT)
#define	IsRegistered(x)		((x)->status >= STAT_SERVER)
#define	IsConnecting(x)		((x)->status == STAT_CONNECTING)
#define	IsHandshake(x)		((x)->status == STAT_HANDSHAKE)
#define	IsMe(x)			((x)->status == STAT_ME)
#define	IsUnknown(x)		((x)->status == STAT_UNKNOWN)
#define	IsServer(x)		((x)->status == STAT_SERVER)
#define	IsClient(x)		((x)->status == STAT_CLIENT)
#define	IsPerson(x)		((x)->user && IsClient(x))
#define	IsPrivileged(x)		(((x)->umode & UMODE_OPER) || IsServer(x))

#define	SetConnecting(x)	((x)->status = STAT_CONNECTING)
#define	SetHandshake(x)		((x)->status = STAT_HANDSHAKE)
#define	SetMe(x)		((x)->status = STAT_ME)
#define	SetUnknown(x)		((x)->status = STAT_UNKNOWN)
#define	SetServer(x)		((x)->status = STAT_SERVER)
#define	SetClient(x)		((x)->status = STAT_CLIENT)

/* Client Flags */
#define	FLAGS_PINGSENT		0x00000001	/* Unreplied ping sent */
#define	FLAGS_DEADSOCKET 	0x00000002	/* Local socket is dead--exiting soon */
#define	FLAGS_KILLED		0x00000004	/* Prevents "QUIT" from being sent for this */
#define	FLAGS_CLOSING		0x00000008	/* set when closing to suppress errors */
#define	FLAGS_GOTID		0x00000010	/* identd check successful */
#define	FLAGS_NONL		0x00000020	/* No \n in buffer */
#define FLAGS_NORMALEX		0x00000040	/* Client exited normally */
#define FLAGS_SENDQEX		0x00000080	/* SendQ exceeded */
#define FLAGS_ULINE		0x00000100	/* client is U-lined */
#define FLAGS_SECURE		0x00000200	/* client is secure */
#define FLAGS_HASHEDIP		0x00000400	/* client is in the ip address hash table */
#define FLAGS_HASHED		0x00000800	/* client is in the client/server hash table */
#define FLAGS_SUIDPAIR		0x00001000	/* cptr->id.id is a valid servid/userid pair */
#define FLAGS_BLOCKED		0x00002000	/* socket is in blocking state */
#define FLAGS_PARSINGRECVQ	0x00004000	/* client has data in recvQ to parse */
#define FLAGS_DCCNOTICE		0x00008000	/* seen notice regarding DCC SENDs */
#define FLAGS_HASVHOST		0x00010000	/* has a virtual host */
#define FLAGS_WANTZIP		0x00020000	/* Link should be zipped */
#define FLAGS_WANTDKEY		0x00040000	/* Link should be encrypted if possible */

#define PingSent(x)		((x)->flags & FLAGS_PINGSENT)
#define DeadSocket(x)		((x)->flags & FLAGS_DEADSOCKET)
#define IsKilled(x)		((x)->flags & FLAGS_KILLED)
#define IsClosing(x)		((x)->flags & FLAGS_CLOSING)
#define GotID(x)		((x)->flags & FLAGS_GOTID)
#define NoNewLine(x)		((x)->flags & FLAGS_NONL)
#define NormalExit(x)		((x)->flags & FLAGS_NORMALEX)
#define SendQEx(x)		((x)->flags & FLAGS_SENDQEX)
#define IsULine(x)		((x)->flags & FLAGS_ULINE)
#define IsSecure(x)		((x)->flags & FLAGS_SECURE)
#define HashedIP(x)		((x)->flags & FLAGS_HASHEDIP)
#define IsHashed(x)		((x)->flags & FLAGS_HASHED)
#define HasSUID(x)		((x)->flags & FLAGS_SUIDPAIR)
#define IsBlocked(x)		((x)->flags & FLAGS_BLOCKED)
#define ParsingRecvQ(x)		((x)->flags & FLAGS_PARSINGRECVQ)
#define SeenDCCNotice(x)	((x)->flags & FLAGS_DCCNOTICE)
#define HasVhost(x)		((x)->flags & FLAGS_HASVHOST)
#define WantZIP(x)		((x)->flags & FLAGS_WANTZIP)
#define WantDKEY(x)		((x)->flags & FLAGS_WANTDKEY)

#define SetPingSent(x)		((x)->flags |= FLAGS_PINGSENT)
#define SetDeadSocket(x)	((x)->flags |= FLAGS_DEADSOCKET)
#define SetKilled(x)		((x)->flags |= FLAGS_KILLED)
#define SetClosing(x)		((x)->flags |= FLAGS_CLOSING)
#define SetGotID(x)		((x)->flags |= FLAGS_GOTID)
#define SetNoNewLine(x)		((x)->flags |= FLAGS_NONL)
#define SetNormalExit(x)	((x)->flags |= FLAGS_NORMALEX)
#define SetSendQEx(x)		((x)->flags |= FLAGS_SENDQEX)
#define SetULine(x)		((x)->flags |= FLAGS_ULINE)
#define SetSecure(x)		((x)->flags |= FLAGS_SECURE)
#define SetHashedIP(x)		((x)->flags |= FLAGS_HASHEDIP)
#define SetHashed(x)		((x)->flags |= FLAGS_HASHED)
#define SetSUID(x)		((x)->flags |= FLAGS_SUIDPAIR)
#define SetBlocked(x)		((x)->flags |= FLAGS_BLOCKED)
#define SetParsingRecvQ(x)	((x)->flags |= FLAGS_PARSINGRECVQ)
#define SetDCCNotice(x)		((x)->flags |= FLAGS_DCCNOTICE)
#define SetVhost(x)		((x)->flags |= FLAGS_HASVHOST)
#define SetWantZIP(x)		((x)->flags |= FLAGS_WANTZIP)
#define SetWantDKEY(x)		((x)->flags |= FLAGS_WANTDKEY)

#define ClearPingSent(x)	((x)->flags &= ~FLAGS_PINGSENT)
#define ClearDeadSocket(x)	((x)->flags &= ~FLAGS_DEADSOCKET)
#define ClearKilled(x)		((x)->flags &= ~FLAGS_KILLED)
#define ClearClosing(x)		((x)->flags &= ~FLAGS_CLOSING)
#define ClearGotID(x)		((x)->flags &= ~FLAGS_GOTID)
#define ClearNoNewLine(x)	((x)->flags &= ~FLAGS_NONL)
#define ClearNormalExit(x)	((x)->flags &= ~FLAGS_NORMALEX)
#define ClearSendQEx(x)		((x)->flags &= ~FLAGS_SENDQEX)
#define ClearULine(x)		((x)->flags &= ~FLAGS_ULINE)
#define ClearSecure(x)		((x)->flags &= ~FLAGS_SECURE)
#define ClearHashedIP(x)	((x)->flags &= ~FLAGS_HASHEDIP)
#define ClearHashed(x)		((x)->flags &= ~FLAGS_HASHED)
#define ClearSUID(x)		((x)->flags &= ~FLAGS_SUIDPAIR)
#define ClearBlocked(x)		((x)->flags &= ~FLAGS_BLOCKED)
#define ClearParsingRecvQ(x)	((x)->flags &= ~FLAGS_PARSINGRECVQ)
#define ClearDCCNotice(x)	((x)->flags &= ~FLAGS_DCCNOTICE)
#define ClearVhost(x)		((x)->flags &= ~FLAGS_HASVHOST)
#define ClearWantZIP(x)		((x)->flags &= ~FLAGS_WANTZIP)
#define ClearWantDKEY(x)	((x)->Flags &= ~FLAGS_WANTDKEY)

/* SERVER-only flags */
#define SERVFLAG_USERBURST	0x00000001	/* Server in nick/server netburst */
#define SERVFLAG_TOPICBURST	0x00000002	/* Server in topic netburst */
#define SERVFLAG_SENTSOB	0x00000004	/* We've sent SOB, just need to send EOB */
#define SERVFLAG_RECVEOB	0x00000008	/* We're waiting for EOB */
#define SERVFLAG_DOINGDKEY	0x00000010	/* We're doing DKEY */
#define SERVFLAG_DKEYIN		0x00000020	/* Got DKEY PUB I */
#define SERVFLAG_DKEYOUT	0x00000040	/* Got DKEY PUB O */
#define SERVFLAG_RC4IN		0x00000080	/* Link input is encrypted */
#define SERVFLAG_RC4OUT		0x00000100	/* Link output is encrypted */
#define SERVFLAG_ZIPIN		0x00000200	/* Link input is zipped */
#define SERVFLAG_ZIPOUT		0x00000400	/* Link output iz zipped */
#define SERVFLAG_MAPPED		0x00000800	/* Already mapped in /MAP */

/* When the source code is finally audited for bad pointers, disable
 * this nasty little hack. We'll then be able to see where we have
 * referenenced the client->serv structure when we shouldn't have done.
 */
#define CLIENT_SERV_STRUCT_PARANOIA

#ifdef CLIENT_SERV_STRUCT_PARANOIA
#define HAS_SERVFLAG(x, y)	(((x)->serv != NULL) && ((x)->serv->flags & (y)))
#define SET_SERVFLAG(x, y)	if ((x)->serv != NULL) (x)->serv->flags |= (y)
#define CLR_SERVFLAG(x, y)	if ((x)->serv != NULL) (x)->serv->flags &= ~(y)
#else
#define HAS_SERVFLAG(x, y)	((x)->serv->flags & (y))
#define SET_SERVFLAG(x, y)	((x)->serv->flags |= (y))
#define CLR_SERVFLAG(x, y)	((x)->serv->flags &= ~(y))
#endif

#define UserBurst(x)		HAS_SERVFLAG(x, SERVFLAG_USERBURST)
#define TopicBurst(x)		HAS_SERVFLAG(x, SERVFLAG_TOPICBURST)
#define SendingBurst(x)		(((x)->serv != NULL) && ((x)->serv->flags & (SERVFLAG_USERBURST|SERVFLAG_TOPICBURST)))
#define SentSOB(x)		HAS_SERVFLAG(x, SERVFLAG_SENTSOB)
#define RecvEOB(x)		HAS_SERVFLAG(x, SERVFLAG_RECVEOB)
#define DoingDKEY(x)		HAS_SERVFLAG(x, SERVFLAG_DOINGDKEY)
#define GotDKEYin(x)		HAS_SERVFLAG(x, SERVFLAG_DKEYIN)
#define GotDKEYout(x)		HAS_SERVFLAG(x, SERVFLAG_DKEYOUT)
#define InputRC4(x)		HAS_SERVFLAG(x, SERVFLAG_RC4IN)
#define OutputRC4(x)		HAS_SERVFLAG(x, SERVFLAG_RC4OUT)
#define InputZIP(x)		HAS_SERVFLAG(x, SERVFLAG_ZIPIN)
#define OutputZIP(x)		HAS_SERVFLAG(x, SERVFLAG_ZIPOUT)
#define IsMapped(x)		HAS_SERVFLAG(x, SERVFLAG_MAPPED)

#define SetUserBurst(x)		SET_SERVFLAG(x, SERVFLAG_USERBURST)
#define SetTopicBurst(x)	SET_SERVFLAG(x, SERVFLAG_TOPICBURST)
#define SetSentSOB(x)		SET_SERVFLAG(x, SERVFLAG_SENTSOB)
#define SetRecvEOB(x)		SET_SERVFLAG(x, SERVFLAG_RECVEOB)
#define SetDoingDKEY(x)		SET_SERVFLAG(x, SERVFLAG_DOINGDKEY)
#define SetDKEYin(x)		SET_SERVFLAG(x, SERVFLAG_DKEYIN)
#define SetDKEYout(x)		SET_SERVFLAG(x, SERVFLAG_DKEYOUT)
#define SetInputRC4(x)		SET_SERVFLAG(x, SERVFLAG_RC4IN)
#define SetOutputRC4(x)		SET_SERVFLAG(x, SERVFLAG_RC4OUT)
#define SetInputZIP(x)		SET_SERVFLAG(x, SERVFLAG_ZIPIN)
#define SetOutputZIP(x)		SET_SERVFLAG(x, SERVFLAG_ZIPOUT)
#define SetMapped(x)		SET_SERVFLAG(x, SERVFLAG_MAPPED)

#define ClearUserBurst(x)	CLR_SERVFLAG(x, SERVFLAG_USERBURST)
#define ClearTopicBurst(x)	CLR_SERVFLAG(x, SERVFLAG_TOPICBURST)
#define ClearSentSOB(x)		CLR_SERVFLAG(x, SERVFLAG_SENTSOB)
#define ClearRecvEOB(x)		CLR_SERVFLAG(x, SERVFLAG_RECVEOB)
#define ClearDoingDKEY(x)	CLR_SERVFLAG(x, SERVFLAG_DOINGDKEY)
#define ClearDKEYin(x)		CLR_SERVFLAG(x, SERVFLAG_DKEYIN)
#define ClearDKEYout(x)		CLR_SERVFLAG(x, SERVFLAG_DKEYOUT)
#define ClearInputRC4(x)	CLR_SERVFLAG(x, SERVFLAG_RC4IN)
#define ClearOutputRC4(x)	CLR_SERVFLAG(x, SERVFLAG_RC4OUT)
#define ClearInputZIP(x)	CLR_SERVFLAG(x, SERVFLAG_ZIPIN)
#define ClearOutputZIP(x)	CLR_SERVFLAG(x, SERVFLAG_ZIPOUT)
#define ClearMapped(x)		CLR_SERVFLAG(x, SERVFLAG_MAPPED)

/* SERVER capabilities */
#define CAP_TS			0x00000001	/* Supports TS (TimeStamps) */
#define CAP_BURST		0x00000002	/* Supports BURST protocol */
#define CAP_UNCONN		0x00000004	/* Supports UNCONNECT */
#define CAP_ULINE		0x00000008	/* Supports U: lines (kludge) */
#define CAP_ZIP			0x00000010	/* Supports ZIP (server<->server compression) */
#define CAP_DKEY		0x00000020	/* Supports DKEY protocol */
#define CAP_SSJ3		0x00000040	/* Supports SSJ3 protocol (Smart SJOIN 3) */
#define CAP_SN2			0x00000080	/* Supports SN2 protocol (SNICK 2) */
#define CAP_VHOST		0x00000100	/* Supports VHOST protocol */
#define CAP_SUID		0x00000200	/* Supports Serv/User IDs */
#define CAP_TOK1		0x00000400	/* Supports tokenised server<->server commands */

#define CAP_ID			(CAP_SUID|CAP_TOK1)
#define DEFAULT_CAPABS		(CAP_BURST|CAP_UNCONN|CAP_ZIP|CAP_SSJ3|CAP_SN2|CAP_VHOST|CAP_SUID|CAP_TOK1)

#define CapTS(x)		((x)->localClient->capabs & CAP_TS)
#define CapNOQUIT(x)		((x)->localClient->capabs & CAP_NOQUIT)
#define CapBURST(x)		((x)->localClient->capabs & CAP_BURST)
#define CapUNCONNECT(x)		((x)->localClient->capabs & CAP_UNCONN)
#define CapZIP(x)		((x)->localClient->capabs & CAP_ZIP)
#define CapDKEY(x)		((x)->localClient->capabs & CAP_DKEY)
#define CapSSJ3(x)		((x)->localClient->capabs & CAP_SSJ3)
#define CapSN2(x)		((x)->localClient->capabs & CAP_SN2)
#define CapVHOST(x)		((x)->localClient->capabs & CAP_VHOST)
#define CapSUID(x)		((x)->localClient->capabs & CAP_SUID)
#define CapTOK1(x)		((x)->localClient->capabs & CAP_TOK1)

#define CapID(x)		(((x)->localClient->capabs & CAP_ID) == CAP_ID)

#define HasMode(x, y)		((x)->umode & (y))
#define AddMode(x, y)		((x)->umode |= (y))
#define DelMode(x, y)		((x)->umode &= ~(y))

#define MaskedHost(x)		(((x)->umode & UMODE_MASKED) ? (x)->user->maskedhost : (x)->host)

#define OFLAG_DIE		0x00000001	/* oper can use DIE command */
#define OFLAG_RESTART		0x00000002	/* oper can use RESTART command */
#define OFLAG_REHASH		0x00000004	/* oper can use REHASH command */
#define OFLAG_KLINE		0x00000008	/* oper can use KLINE command */
#define OFLAG_UNKLINE		0x00000010	/* oper can use UNKLINE command */
#define OFLAG_ZAP		0x00000020	/* oper can use ZLINE command */
#define OFLAG_LROUTE		0x00000040	/* oper can do local routing */
#define OFLAG_GROUTE		0x00000080	/* oper can do global routing */
#define OFLAG_LKILL		0x00000100	/* oper can do local kills */
#define OFLAG_GKILL		0x00000200	/* oper can do global kills */
#define OFLAG_LOCOPS		0x00000400	/* oper can send LOCOPS notices */
#define OFLAG_GLOBOPS		0x00000800	/* oper can send GLOBOPS notices */
#define OFLAG_WALLOPS		0x00001000	/* oper can send WALLOPS notices */
#define OFLAG_LNOTICE		0x00002000	/* oper can send local notices */
#define OFLAG_GNOTICE		0x00004000	/* oper can send global notices */
#define OFLAG_FNOTICE		0x00008000	/* oper can receive flood notices */
#define OFLAG_LCLICONN		0x00010000	/* oper can receive local cliconnect notices */
#define OFLAG_GCLICONN		0x00020000	/* oper can receive global cliconnect notices */
#define OFLAG_NETADMIN		0x00040000	/* oper is a Network Administrator */
#define OFLAG_ADMIN		0x00080000	/* oper is a Server Administrator */
#define OFLAG_COADMIN		0x00100000	/* oper is a Server Co-Administrator */
#define OFLAG_SADMIN		0x00200000	/* oper is a Services Administrator */
#define OFLAG_RSTAFF		0x00400000	/* oper is routing staff (can see routing notices and server ip's) */
#define OFLAG_OPERMOTD		0x00800000	/* oper receives OPERMOTD on oper-up */
#define OFLAG_NORECVQTHROTTLE	0x01000000	/* oper is except from recvQ throttling */

#define OPHasFlag(x, y)		((x)->localUser->oflags & (y))
#define OPAddFlag(x, y)		((x)->localUser->oflags |= (y))
#define OPDelFlag(x, y)		((x)->localUser->oflags &= ~(y))

enum {
	CCONN_LEV = 1,		/* Cli-connect notices */
	REJ_LEV,		/* Reject notices */
	SKILL_LEV,		/* Server kill notices */
	SPY_LEV,		/* Spying notices */
	DEBUG_LEV,		/* Debug notices */
	FLOOD_LEV,		/* Flood notices */
	SPAM_LEV,		/* Spambot notices */
	DCCSEND_LEV,		/* DCCAllow notices */
	GCCONN_LEV		/* Global cli-connect notices */
};

enum {
	DEBUG_FATAL = 1,	/* Fatal errors */
	DEBUG_ERROR,		/* Errors */
	DEBUG_NOTICE,		/* Notices */
	DEBUG_DNS,		/* DNS debug */
	DEBUG_INFO,		/* General info debug */
	DEBUG_NUM,		/* Numeric debug */
	DEBUG_SEND,		/* Sending debug */
	DEBUG_DEBUG,		/* Debugging (unimportant) debug */
	DEBUG_MALLOC,		/* Memory debug */
	DEBUG_LIST		/* List debug */
};

enum {
	MFTYPE_MOTD,		/* MOTD */
	MFTYPE_OPERMOTD,	/* OPERMOTD */
	MFTYPE_RULES		/* RULES */
};

enum {
	AUTH_PLAINTEXT,
	AUTH_DES_56,
	AUTH_MD5_56,
	AUTH_MD5_128,
	AUTH_SHA1_160,
	AUTH_RIPEMD_160
};

struct Counter {
	int server;		/* servers */
	int myserver;		/* my servers */
	int myulined;		/* my ulined servers */
	int oper;		/* opers */
	int chan;		/* channels */
	int local;		/* local clients */
	int total;		/* total clients */
	int invisi;		/* invisible clients */
	int unknown;		/* unknown connections */
	int max_loc;		/* max local clients */
	int max_tot;		/* max global clients */
	int cli_restart;	/* clients since (re)start */
	time_t start;		/* when we started collecting info */
	u_long today;		/* client connections today */
	time_t day;		/* when today started */
	u_long weekly;		/* connections this week */
	time_t week;		/* when this week started */
	u_long monthly;		/* connections this month */
	time_t month;		/* when this month started */
	u_long yearly;		/* connections this year (this is gunna be be big!) */
	time_t year;		/* when this year started */
};

struct Whowas {
	int hashv;
	char name[NICKLEN + 1];
	char username[USERLEN + 1];
	char hostname[HOSTLEN + 1];
	char maskedhost[HOSTLEN + 1];
	char realname[REALLEN + 1];
	char *servername;
	time_t logoff;
	aClient *online;
	aWhowas *next, *prev;
	aWhowas *cnext, *cprev;
};

/* iphash entry structure */
struct iphash_entry {
	struct in_addr ip;
	int count;
	IPEntry *next;
};

/* module structure */
struct module {
	char *name;		/* Used directly: do not move! */
	char *info;		/* Used directly: do not move! */
	int version;		/* Used directly: do not move! */
	char *revision;		/* Used directly: do not move! */
	void *ptr;
	int (*unloadfunc)();
	unsigned short flags;
	dlink_list commands;
	dlink_list events;
	dlink_list hookevents;
	dlink_list hooks;
	dlink_list usermodes;
};

#if (defined(STATIC_MODULES) && defined(DYNAMIC_WRAPPER)) || !defined(STATIC_MODULES)
struct module_depend {
	char *symbol;
	void **ptr;
	char *source;
};

struct module_delayed {
	char *name;
	int retries;
};
#endif

struct event {
	char *name;
	void (*func)();
	void *arg;
	time_t freq;
	time_t when;
	unsigned loop : 1;
	Module *owner;
};

/* ServerCache structure */
struct scache_entry {
	char name[HOSTLEN + 1];
	ServerCache *next;
};

#ifdef USE_THROTTLE
struct hashent_t {
	void *ent;
	SLIST_ENTRY(hashent_t) lp;
};

struct hash_table_t {
	int size;
	hashent_list *table;
	size_t keyoffset;
	size_t keylen;
	struct {
		unsigned nocase : 1;
		unsigned string : 1;
	} flag;
	int (*cmpfunc)(void *, void *);
};

struct throttle_t {
	char addr[HOSTIPLEN + 1];
	int conns;
	time_t first;
	time_t last;
	time_t zline_start;
	int stage;
	int re_zlines;
	LIST_ENTRY(throttle_t) lp;
};
#endif

struct listener {
	ConfigItem item;
	char ipaddr[HOSTIPLEN + 1];
	int port;
	unsigned short flags;
	int fd;
	unsigned long count;
	int clients;
	struct in_addr ip;
	time_t lasttime;
	long sendM;
	long receiveM;
	long sendK;
	long receiveK;
	unsigned short sendB;
	unsigned short receiveB;
	ConfigItem_listen *conf;
};

struct dns_query {
	void *ptr;
	adns_query query;
	adns_answer answer;
	void (*callback)();
};

struct connauth {
	aClient *cptr;
	int fd;
	time_t timeout;
	struct {
		unsigned ident_connect : 1;
		unsigned ident_pending : 1;
		unsigned dns_pending : 1;
	} flag;
};

struct message_file {
	char *filename;
	short type;
	dlink_list data;
	char changed_date[MAX_DATE_STRING + 1];
};

struct suid {
	unsigned int id;
	char string[8];
};

struct capab_t {
	unsigned int flag;
	char *token;
	unsigned link_state : 1;
	unsigned synch_state : 1;
	char *pretty_token;
};

struct ircd_t {
	char *conf_file;
	int conf_errors;
	unsigned valid_conf : 1;
	char *debug_mode;
	int debug_level;
	unsigned run_in_terminal : 1;
	unsigned do_ban_check : 1;
	unsigned do_rehash : 1;
	time_t next_event;
	time_t next_expire_klines;
	int max_con_count;
	int max_cli_count;
	float curr_sendK;
	float curr_recvK;
	unsigned rehashing : 1;
	struct timeval ircd_time;
	unsigned int default_capabs;
	char masking_keys[256];

#ifdef DEBUGMODE
	struct {
		int read_calls;
		int read_errors;
		int read_errors_ignored;
		int write_calls;
		int write_errors;
		int write_errors_ignored;
	} debug;
#endif
};

struct auth {
	char *string;
	short type;
};

struct Server {
	char *up;
	char by_nick[NICKLEN + 1];
	char by_user[USERLEN + 1];
	char by_host[HOSTLEN + 1];
#ifdef USE_OPENSSL
	void *sessioninfo_in;		/* pointer to opaque sessioninfo structure */
	void *sessioninfo_out;		/* pointer to opaque sessioninfo structure */
	void *rc4_in;			/* etc */
	void *rc4_out;			/* etc */
#endif
	void *zip_out;			/* inbound zipped data */
	void *zip_in;			/* outbound zipped data */
	ConfigItem_link *conf;		/* config */
	unsigned long flags;		/* Server-only flags */
	aClient *userid[ID_MAP_SIZE];
};

struct User_local {
	time_t last;
	unsigned long oflags;
	ConfigItem_oper *oper;
	ConfigItem_allow *allow;
	SLink *invited;
	LOpts *lopt;
	int nick_changes;
	time_t last_nick_change;
	int away_changes;
	time_t last_away_change;
#ifdef ANTI_SPAMBOT
	time_t last_join_time;
	time_t last_part_time;
	int join_part_count;
	int oper_warn_countdown;
#endif
#ifdef FLUD
	time_t fludblock;
	struct fludbot *fluders;
#endif
};

struct User {
	char maskedhost[HOSTLEN + 1];
	chanMember *channel;
	char *away;
	int joined;
	char *server;
	unsigned int servicestamp;
	SLink *silence;
	dlink_list dccallow_list;
	dlink_list on_dccallow_list;
	dlink_list accept_list;
	dlink_list on_accept_list;
	time_t last_accept_notice;
	time_t first_message_time;
	int number_of_messages;
	unsigned flood_notice : 1;
	SLink *watch;
	int watches;
	aWhowas *whowas;
	char nicksent;
#ifdef FLUD
	SLink *fludees;
#endif
};

struct Client_local {
	dlink_node self;
	SBuf sendQ;
	SBuf recvQ;
	short last_sendq;
	char buffer[BUFSIZE];
	long sendM;
	long receiveM;
	long sendK;
	long receiveK;
	unsigned short sendB;
	unsigned short receiveB;
	int fd;
#ifdef USE_OPENSSL
	SSL *ssl;
#endif
	int count;
	char sockhost[HOSTLEN + 1];
	char passwd[PASSWDLEN + 1];
	unsigned int capabs;
	DNSQuery *dns_query;
	int sockerr;
	Listener *listener;
	ConfigItem_class *class;
};

struct Client {
	aClient *next, *prev, *hnext, *idhnext;
	LocalClient *localClient;
	anUser *user;
	LocalUser *localUser;
	aServer *serv;
	time_t lasttime;
	time_t firsttime;
	time_t since;
	time_t tsinfo;
	unsigned long flags;
	unsigned long umode;
	aClient *from;
	aClient *uplink;
  	int hopcount;
	short status;
	char name[HOSTLEN + 1];
	char username[USERLEN + 1];
	char host[HOSTLEN + 1];
	char info[REALLEN + 1];
	struct in_addr ip;
	SUID id;
	char hostip[HOSTIPLEN + 1];
	unsigned int serial;
};

/* statistics structures */
struct stats {
	unsigned int is_cl;     /* number of client connections */
	unsigned int is_sv;     /* number of server connections */
	unsigned int is_ni;     /* connection but no idea who it was */
	unsigned short is_cbs;  /* bytes sent to clients */
	unsigned short is_cbr;  /* bytes received to clients */
	unsigned short is_sbs;	/* bytes sent to servers */
	unsigned short is_sbr;  /* bytes received to servers */
	unsigned long is_cks;	  /* k-bytes sent to clients */
	unsigned long is_ckr;	  /* k-bytes received to clients */
	unsigned long is_sks;	  /* k-bytes sent to servers */
	unsigned long is_skr;	  /* k-bytes received to servers */
	time_t      is_cti;		  /* time spent connected by clients */
	time_t      is_sti;		  /* time spent connected by servers */
	unsigned int is_ac;		  /* connections accepted */
	unsigned int is_ref;	  /* accepts refused */
	unsigned int is_unco;	  /* unknown commands */
	unsigned int is_wrdi;	  /* command going in wrong direction */
	unsigned int is_unpf;	  /* unknown prefix */
	unsigned int is_empt;	  /* empty message */
	unsigned int is_num;	  /* numeric message */
	unsigned int is_kill;  	/* number of kills generated on collisions */
	unsigned int is_fake;	  /* MODE 'fakes' */
	unsigned int is_asuc;	  /* successful auth requests */
	unsigned int is_abad;	  /* bad auth requests */
	unsigned int is_udp;		/* packets recv'd on udp port */
#ifdef FLUD
	unsigned int is_flud;	  /* users/channels flood protected */
#endif	                  /* FLUD */
};

/* mode structure for channels */
struct SMode {
	unsigned int mode;
	int limit;
	char key[KEYLEN + 1];
};

/* channel member link structure, used for chanmember chains */
struct channelMember {
	chanMember *nextuser;
	chanMember *nextchan;
	aClient *cptr;
	aChannel *chptr;
	int flags;
	int bans;
};

/* channel ban link structure */
struct channelban {
	char *id;
	char *set_by;
	time_t when;
};

/* general link structure used for chains */
struct slink {
	SLink *next;
	union {
		aClient *cptr;
		aChannel *chptr;
		channelBan *banptr;
		aWatch *wptr;
		char *cp;
	} value;
	int flags;
};

/* channel structure */

struct Channel {
	struct Channel *nextch, *prevch, *hnextch;
	int hashv;
	Mode mode;
	char topic[TOPICLEN + 1];
        char topic_nick[NICKLEN + 1];
	time_t topic_time;
	int users;
	chanMember *members;
	SLink *invites;
	dlink_list banlist;
	dlink_list exceptlist;
	dlink_list invexlist;
	time_t channelts;
#ifdef FLUD
	time_t fludblock;
	struct fludbot *fluders;
#endif
	char chname[CHANNELLEN+1];
	time_t last_knock;
	time_t first_message_time;
	int number_of_messages;
	unsigned flood_notice : 1;
};

#define TS_CURRENT	5		/* Current TS protocol version */
#define TS_MIN		3		/* Minimum TS protocol version */

#define	HoldChannel(x)		(!(x))
#define	SecretChannel(x)	((x) && ((x)->mode.mode & CMODE_SECRET))
#define	ShowChannel(v,c)	(PubChannel(c) || IsMember((v),(c)))
#define	PubChannel(x)		((!x) || ((x)->mode.mode & CMODE_SECRET) == 0)
#define IsMember(blah, chan)	((blah && blah->user && find_user_member((blah->user)->channel, chan)) ? 1 : 0)
#define	IsChannelName(name)	((name) && (*(name) == '#' || *(name) == '&'))

/* Misc macros */

#define	BadPtr(x)	((x) == NULL || (*(x) == '\0'))
#define	isvalid(c)	(((c) >= 'A' && (c) < '~') || isdigit(c) || (c) == '-')
#define MyConnect(x)	((x)->localClient != NULL)
#define MyClient(x)	(MyConnect(x) && IsClient(x))

#define IsSendable(x)	(!((x)->flags & FLAGS_BLOCKED) && SBufLength(&x->localClient->sendQ) < 16384)
#define DoList(x)	(((x)->localUser != NULL) && ((x)->localUser->lopt != NULL))

#define strncpyzt(x, y, N)	do { strncpy(x, y, N); x[N - 1] = '\0'; } while (0)

/* return values for hunt_server() */

#define	HUNTED_NOSUCH	(-1)	/* if the hunted server is not found */
#define	HUNTED_ISME	0	/* if this server should execute the command */
#define	HUNTED_PASS	1	/* if message passed onwards successfully */

/* used when sending to #mask or $mask */

#define	MATCH_SERVER  1
#define	MATCH_HOST    2

/* misc variable externs */

extern char ircd_version[128], *revision;
extern char *infotext[], *dalinfotext[];
extern char *generation, *creation;

/* misc defines */

#define ZIP_NEXT_BUFFER -4
#define RC4_NEXT_BUFFER -3
#define	FLUSH_BUFFER	-2

/* client authentication return values */
#define CLIENTAUTH_ERRMSG "Unauthorized client connection from %s%s"
enum {
	CLIENTAUTH_MATCHED,	/* access granted, I:line attached */
	CLIENTAUTH_NOMATCH,	/* no matched I:line */
	CLIENTAUTH_ILINEFULL,	/* I:line full */
	CLIENTAUTH_INVALIDPW,	/* Invalid password */
	CLIENTAUTH_TOOMANYIPS	/* Too many clients from single IP */
};

#ifdef FLUD
struct fludbot {
	struct Client *fluder;
	int         count;
	time_t      first_msg, last_msg;
	struct fludbot *next;
};

#endif /* FLUD */

struct Watch {
	   aWatch  *hnext;
	   time_t   lasttime;
	   SLink  *watch;
	   char  nick[1];
};

struct ListOptions {
	   LOpts *next;
	   SLink  *yeslist, *nolist;
	   int   starthash;
	   short int   showall;
	   unsigned short usermin;
	   int   usermax;
	   time_t   currenttime;
	   time_t   chantimemin;
	   time_t   chantimemax;
	   time_t   topictimemin;
	   time_t   topictimemax;
};

/* internal defines for cptr->sockerr */
#define IRCERR_BUFALLOC		-11
#define IRCERR_ZIP		-12
#define IRCERR_SSL		-13

#endif /* __struct_include__ */
