/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/aliases.c: services' aliases module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: aliases.c,v 1.31 2004/05/13 18:15:31 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include "modules.h"
#include "hook.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

Module MOD_HEADER(aliases) = {
	"modules/aliases.so",
	"services aliases",
	6, "$Revision: 1.31 $"
};

int MOD_LOAD(aliases)()
{
	register_command(&MOD_HEADER(aliases), &CMD_CHANSERV, m_chanserv);
	register_command(&MOD_HEADER(aliases), &CMD_NICKSERV, m_nickserv);
	register_command(&MOD_HEADER(aliases), &CMD_OPERSERV, m_operserv);
	register_command(&MOD_HEADER(aliases), &CMD_MEMOSERV, m_memoserv);
	register_command(&MOD_HEADER(aliases), &CMD_STATSERV, m_statserv);
	register_command(&MOD_HEADER(aliases), &CMD_HOSTSERV, m_hostserv);
	register_command(&MOD_HEADER(aliases), &CMD_BOTSERV, m_botserv);

	register_command(&MOD_HEADER(aliases), &CMD_CS, m_chanserv);
	register_command(&MOD_HEADER(aliases), &CMD_NS, m_nickserv);
	register_command(&MOD_HEADER(aliases), &CMD_OS, m_operserv);
	register_command(&MOD_HEADER(aliases), &CMD_MS, m_memoserv);
	register_command(&MOD_HEADER(aliases), &CMD_SS, m_statserv);
	register_command(&MOD_HEADER(aliases), &CMD_HS, m_hostserv);
	register_command(&MOD_HEADER(aliases), &CMD_BS, m_botserv);

	register_command(&MOD_HEADER(aliases), &CMD_SERVICES, m_services);
	register_command(&MOD_HEADER(aliases), &CMD_IDENTIFY, m_identify);

	return MOD_SUCCESS;
}

int MOD_UNLOAD(aliases)()
{
	return MOD_SUCCESS;
}

static inline int do_alias(aClient *sptr, int parc, char *parv[], char *client_server, char *client_nick)
{
	aClient *acptr = NULL;

	if (BadPtr(client_server)) {
		send_me_numeric(sptr, ERR_FUNCDISABLED, client_nick);
		return 0;
	}
	if (parc < 2 || *parv[1] == '\0') {
		send_me_numericNA(sptr, ERR_NOTEXTTOSEND);
		return 0;
	}
	if ((acptr = find_person_on_server(client_nick, client_server)) == NULL) {
		send_me_numeric(sptr, ERR_SERVICESDOWN, client_nick);
		return 0;
	}

	sendto_one_client_onserver_copy(acptr, sptr, &CMD_PRIVMSG_D, PREFIX_COLON, parv[1]);
	return 0;
}

/*
 * m_*serv
 *	parv[0] = sender prefix
 *	parv[1] = message to relay to service
 */
int m_chanserv(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	return do_alias(sptr, parc, parv, NetworkConfig.services_server, CHANSERV);
}

int m_nickserv(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	return do_alias(sptr, parc, parv, NetworkConfig.services_server, NICKSERV);
}

int m_memoserv(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	return do_alias(sptr, parc, parv, NetworkConfig.services_server, MEMOSERV);
}

int m_operserv(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	return do_alias(sptr, parc, parv, NetworkConfig.services_server, OPERSERV);
}

int m_statserv(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	return do_alias(sptr, parc, parv, NetworkConfig.stats_server, STATSERV);
}

int m_hostserv(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	return do_alias(sptr, parc, parv, NetworkConfig.services_server, HOSTSERV);
}

int m_botserv(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	return do_alias(sptr, parc, parv, NetworkConfig.services_server, BOTSERV);
}

/*
 * m_services
 *	parv[0] = sender prefix
 *	parv[1] = message to relay to service
 */
int m_services(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	char *tmp = NULL;

	if (BadPtr(NetworkConfig.services_server)) {
		send_me_numeric(sptr, ERR_FUNCDISABLED, "SERVICES");
		return 0;
	}
	if (parc < 2 || *parv[1] == '\0') {
		send_me_numericNA(sptr, ERR_NOTEXTTOSEND);
		return 0;
	}
	if (!mycmp(parv[1], "help")) {
		send_me_noticeNA(sptr, ":For ChanServ help, use /chanserv help");
		send_me_noticeNA(sptr, ":For NickServ help, use /nickserv help");
		send_me_noticeNA(sptr, ":For MemoServ help, use /memoserv help");
		return 0;
	}
	if ((tmp = strchr(parv[1], ' ')) != NULL) {
		while (*tmp == ' ') {
			tmp++;
		}
		if (*tmp == '#') {
			return m_chanserv(cptr, sptr, parc, parv);
		}
	}
	return m_nickserv(cptr, sptr, parc, parv);
}

/*
 * m_identify
 *	parv[0] = sender prefix
 *	parv[1] = message to relay to service
 */
int m_identify(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	aClient *acptr = NULL;
	int gate = 0;

	if (BadPtr(NetworkConfig.services_server)) {
		send_me_numeric(sptr, ERR_FUNCDISABLED, "IDENTIFY");
		return 0;
	}
	if (parc < 2 || *parv[1] == '\0') {
		send_me_numericNA(sptr, ERR_NOTEXTTOSEND);
		return 0;
	}
	if (*parv[1] == '#' && strchr(parv[1], ' ') != NULL) {
		acptr = find_person_on_server(CHANSERV, NetworkConfig.services_server);
		gate = 1;
	}
	else {
		acptr = find_person_on_server(NICKSERV, NetworkConfig.services_server);
		gate = 0;
	}
	if (acptr == NULL) {
		send_me_numeric(sptr, ERR_SERVICESDOWN, gate ? CHANSERV : NICKSERV);
		return 0;
	}

	sendto_one_client_onserver(acptr, sptr, &CMD_PRIVMSG_D, ":IDENTIFY %s", parv[1]);
	return 0;
}
