/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/antidrone.c: drone protection
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: antidrone.c,v 1.14 2004/05/08 16:12:02 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include "hook.h"
#include "modules.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

static int anti_drone(HookData *);

Module MOD_HEADER(antidrone) = {
	"modules/antidrone.so",
	"drone protection",
	6, "$Revision: 1.14 $"
};

int MOD_LOAD(antidrone)()
{
	register_hook_event(&MOD_HEADER(antidrone), h_post_access, anti_drone);
	return MOD_SUCCESS;
}

int MOD_UNLOAD(antidrone)()
{
	return MOD_SUCCESS;
}

static void block_drone(aClient *cptr, aClient *sptr, char *type)
{
	ircstp->is_ref++;
	sendto_realops_lev(REJ_LEV, "Rejecting possible %s drone: %s!%s@%s (%s)", type, sptr->name,
		sptr->username, sptr->host, sptr->info);
#ifdef USE_THROTTLe
	if (FloodConfig.throttle_rejected_clients) {
		throttle_force(sptr->host);
	}
#endif
	exit_client(cptr, sptr, sptr, "You match the pattern of a known trojan, please "
		"check your system.");
	return;
}

static int detect_fizzer(aClient *sptr, char *username)
{
	char tmp[REALLEN + 1], *buf[REALLEN + 1], ubuf[USERLEN + 1], *s, *p;
	int scnt = 0, cnt = 0, ulen = 0, ulen_given = strlen(username);

	strncpyzt(tmp, sptr->info, REALLEN + 1);
	for (s = tmp; *s != '\0'; s++) {
		if (*s == ' ') {
			scnt++;
		}
	}
	if (!scnt) {
		return 0;
	}

	for (s = strtoken(&p, tmp, " "); s != NULL; s = strtoken(&p, NULL, " ")) {
		if (scnt < 0) {
			break;
		}
		buf[scnt--] = s;
		cnt++;
	}

	scnt = 0;
	while (cnt) {
		if (ulen >= USERLEN) {
			break;
		}
		for (s = buf[scnt]; *s != '\0' && ulen <= USERLEN; s++) {
			ubuf[ulen++] = *s;
		}
		scnt++;
		cnt--;
	}

	ubuf[ulen] = '\0';
	if (ulen > ulen_given) {
		ubuf[ulen_given] = '\0';
	}

	if (!strcmp(username, ubuf)) {
		return 1;
	}
	return 0;
}

static int detect_acebot(aClient *sptr, char *username)
{
	unsigned char *s;

	if (strlen(username) <= 2) {
		return 0;
	}
	for (s = (unsigned char *)sptr->username; *s != '\0'; s++) {
		if (*s < 'A' || *s > 'Z') {
			return 0;
		}
	}
	if (irccmp(sptr->username, sptr->info)) {
		return 0;
	}
	return 1;
}

static int anti_drone(HookData *hdata)
{
	aClient *cptr = hdata->cptr;
	aClient *sptr = hdata->sptr;
	char *username = (*sptr->username == '~') ? &sptr->username[1] : sptr->username;

	if (detect_fizzer(sptr, username)) {
		block_drone(cptr, sptr, "fizzer-worm");
		return FLUSH_BUFFER;
	}
	if (detect_acebot(sptr, username)) {
		block_drone(cptr, sptr, "acebot-style");
		return FLUSH_BUFFER;
	}
	return 0;
}
