/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/commands.c: commands wrapper
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: commands.c,v 1.34 2004/05/13 16:12:53 rageircd Exp $
 */

#ifndef STATIC_MODULES
#error commands.c cannot be compiled without -DSTATIC_MODULES.
#endif

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include "modules.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

#ifdef DYNAMIC_WRAPPER
Module module_header = {
#else
Module commands_header = {
#endif
	"modules/commands.so",
	"commands wrapper",
	6, "$Revision: 1.34 $"
};

#ifdef DYNAMIC_WRAPPER
int module_load()
#else
int commands_load()
#endif
{
	/* Priority 1: load modules with exported symbols first. */
	m_join_load();
	m_message_load();
	m_mode_load();
	m_nick_load();

	/* And then everything else. */
	m_accept_load();
	m_admin_load();
	m_akill_load();
	m_away_load();
	m_burst_load();
	m_capab_load();
	m_chatops_load();
	m_close_load();
	m_connect_load();
	m_dalinfo_load();
	m_dccallow_load();
	m_die_load();
	m_dkey_load();
	m_error_load();
	m_globops_load();
	m_gnotice_load();
	m_info_load();
	m_invite_load();
	m_ison_load();
	m_kick_load();
	m_kill_load();
	m_kline_load();
	m_knock_load();
	m_links_load();
	m_list_load();
	m_locops_load();
	m_lusers_load();
	m_map_load();
	m_mkpasswd_load();
	m_motd_load();
	m_myid_load();
	m_names_load();
	m_oper_load();
	m_opermotd_load();
	m_part_load();
	m_pass_load();
	m_ping_load();
	m_pong_load();
	m_quit_load();
	m_rakill_load();
	m_rehash_load();
	m_restart_load();
	m_rules_load();
	m_sajoin_load();
	m_sakick_load();
	m_samode_load();
	m_server_load();
	m_sgline_load();
	m_silence_load();
	m_sjoin_load();
	m_snick_load();
	m_sqline_load();
	m_squit_load();
	m_stats_load();
	m_svinfo_load();
	m_svskill_load();
	m_svsmode_load();
	m_svsnick_load();
	m_svsnoop_load();
	m_szline_load();
	m_time_load();
	m_topic_load();
	m_trace_load();
	m_unkline_load();
	m_unsgline_load();
	m_unsqline_load();
	m_unszline_load();
	m_user_load();
	m_userhost_load();
	m_version_load();
	m_vhost_load();
	m_wallops_load();
	m_watch_load();
	m_who_load();
	m_whois_load();
	m_whowas_load();
	m_zline_load();

#ifdef DYNAMIC_WRAPPER
	module_header.flags |= MOD_FLAG_PERM;
#else
	commands_header.flags |= MOD_FLAG_PERM;
#endif

	return MOD_SUCCESS;
}

#define CLR(xx) clear_objects(&xx##_header); xx##_unload()

#ifdef DYNAMIC_WRAPPER
int module_unload()
#else
int commands_unload()
#endif
{
	/* Priority: unload modules with resolved symbols first. */
	CLR(m_accept);
	CLR(m_dccallow);
	CLR(m_samode);
	CLR(m_names);

	/* And then everything else. */
	CLR(m_admin);
	CLR(m_akill);
	CLR(m_away);
	CLR(m_burst);
	CLR(m_capab);
	CLR(m_chatops);
	CLR(m_close);
	CLR(m_connect);
	CLR(m_dalinfo);
	CLR(m_die);
	CLR(m_dkey);
	CLR(m_error);
	CLR(m_globops);
	CLR(m_gnotice);
	CLR(m_info);
	CLR(m_invite);
	CLR(m_ison);
	CLR(m_join);
	CLR(m_kick);
	CLR(m_kill);
	CLR(m_kline);
	CLR(m_knock);
	CLR(m_links);
	CLR(m_list);
	CLR(m_locops);
	CLR(m_lusers);
	CLR(m_map);
	CLR(m_message);
	CLR(m_mkpasswd);
	CLR(m_mode);
	CLR(m_motd);
	CLR(m_myid);
	CLR(m_names);
	CLR(m_nick);
	CLR(m_oper);
	CLR(m_opermotd);
	CLR(m_part);
	CLR(m_pass);
	CLR(m_ping);
	CLR(m_pong);
	CLR(m_quit);
	CLR(m_rakill);
	CLR(m_rehash);
	CLR(m_restart);
	CLR(m_rules);
	CLR(m_sajoin);
	CLR(m_sakick);
	CLR(m_server);
	CLR(m_sgline);
	CLR(m_silence);
	CLR(m_sjoin);
	CLR(m_snick);
	CLR(m_sqline);
	CLR(m_squit);
	CLR(m_stats);
	CLR(m_svinfo);
	CLR(m_svskill);
	CLR(m_svsmode);
	CLR(m_svsnick);
	CLR(m_svsnoop);
	CLR(m_szline);
	CLR(m_time);
	CLR(m_topic);
	CLR(m_trace);
	CLR(m_unkline);
	CLR(m_unsgline);
	CLR(m_unsqline);
	CLR(m_unszline);
	CLR(m_user);
	CLR(m_userhost);
	CLR(m_version);
	CLR(m_vhost);
	CLR(m_wallops);
	CLR(m_watch);
	CLR(m_who);
	CLR(m_whois);
	CLR(m_whowas);
	CLR(m_zline);

	return MOD_SUCCESS;
}
