/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_akill.c: AKILL command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_akill.c,v 1.27 2004/05/12 12:28:05 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include "modules.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

extern void send_globops(char *, ...);

Module MOD_HEADER(m_akill) = {
	"modules/commands/m_akill.so",
	"/AKILL command",
	5, "$Revision: 1.27 $"
};

int MOD_LOAD(m_akill)()
{
	register_command(&MOD_HEADER(m_akill), &CMD_AKILL, m_akill);
	return MOD_SUCCESS;
}

int MOD_UNLOAD(m_akill)()
{
	return MOD_SUCCESS;
}

/*
 * m_akill
 *	parv[0] = sender prefix
 *	parv[1] = host
 *	parv[2] = user
 *	parv[3] = length
 *	parv[4] = akiller
 *	parv[5] = time set
 *	parv[6] = reason
 */
int m_akill(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	ConfigItem_ban *ban;
	char *user, *host, *userhost, *akiller, *current_time, *reason, buffer[1024];
	time_t length = 0, timeset = 0;

	if (!IsServer(sptr) || (parc < 6)) {
		return 0;
	}
	if (!IsULine(sptr)) {
		sendto_serv_msg_butone(NULL, &me, NULL, &CMD_GLOBOPS, ":Non-ULined server "
			"%s is trying to AKILL!", parv[0]);
		send_globops("Non-ULined server %s is tring to AKILL!", parv[0]);
		return 0;
	}

	user = parv[2];
	host = parv[1];
	akiller = parv[4];

	length = atol(parv[3]);
	if (!length) {
		length = (86400 * 7);
	}

	timeset = atol(parv[5]);
	if ((timeset + length) <= timeofday) {
		return 0;
	}

	reason = !BadPtr(parv[6]) ? parv[6] : "no reason";
	if (strlen(reason) > 250) {
		reason[250] = '\0';
	}

	current_time = smalldate((time_t)timeset);

	userhost = make_user_host(user, host);
	if ((ban = find_ban(BAN_USERHOST|BAN_SVS, userhost)) == NULL) {
		ircdlog(LOG_BAN, "AUTOKILL on %s (%s) added by %s", userhost, reason, get_client_name(sptr, SHOW_IP));
		ircsprintf(buffer, "%s (%s)", reason, current_time);

		ban = (ConfigItem_ban *)MyMalloc(sizeof(ConfigItem_ban));
		ban->flags |= (BAN_USERHOST|BAN_SVS);
		DupString(ban->mask, userhost);
		DupString(ban->reason, buffer);
		dlink_add(&conf_ban_list, ban);
	}

	sendto_serv_msg_butone(cptr, sptr, NULL, &CMD_AKILL, "%s %s %d %s %d :%s", host, user,
		length, akiller, timeset, reason);
	Internal.do_ban_check = 1;
	return 0;
}
