/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_capab.c: CAPAB protocol command
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_capab.c,v 1.41 2004/04/08 20:54:22 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include "modules.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

Module MOD_HEADER(m_capab) = {
	"modules/commands/m_capab.so",
	"CAPAB protocol",
	5, "$Revision: 1.41 $"
};

int MOD_LOAD(m_capab)()
{
	register_command(&MOD_HEADER(m_capab), &CMD_CAPAB, m_capab);
	return MOD_SUCCESS;
}

int MOD_UNLOAD(m_capab)()
{
	return MOD_SUCCESS;
}

/*
 * m_capab
 *	parv[0] = sender prefix
 *	parv[n] = capabilities
 */
int m_capab(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	Capability *c;
	int i;

	if (cptr != sptr) {
		return 0;
	}

	ASSERT(cptr->localClient != NULL);

	i = cptr->localClient->capabs;
	i &= ~CAP_TS;
	if (i) {
		ircdlog(LOG_ERROR, "Ignoring extra CAPAB (parc %d)", parc);
		return 0;
	}

	for (i = 1; i < parc; i++) {
		for (c = cap_table; c->flag; c++) {
			if (c->token == NULL) {
				continue;
			}
			if (!irccmp(c->token, parv[i])) {
				sptr->localClient->capabs |= c->flag;
				break;
			}
		}
	}

	if ((CapSUID(sptr) && !CapTOK1(sptr)) || (!CapSUID(sptr) && CapTOK1(sptr))) {
		return exit_client(sptr, sptr, &me, "Requires both SUID and TOK1 capability");
	}
	if (!CapSSJ3(sptr)) {
		return exit_client(sptr, sptr, &me, "Requires SSJ3 capability");
	}
	return 0;
}
