/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_chatops.c: CHATOPS command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_chatops.c,v 1.19 2004/03/07 16:13:35 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include "modules.h"
#include "xmode.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

Module MOD_HEADER(m_chatops) = {
	"modules/commands/m_chatops.so",
	"/CHATOPS command",
	5, "$Revision: 1.19 $"
};

int MOD_LOAD(m_chatops)()
{
	register_command(&MOD_HEADER(m_chatops), &CMD_CHATOPS, m_chatops);
	return MOD_SUCCESS;
}

int MOD_UNLOAD(m_chatops)()
{
	return MOD_SUCCESS;
}

/*
 * m_chatops
 *	parv[0] = sender prefix
 *	parv[1] = message
 */
int m_chatops(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	if (MyClient(sptr) && !HasMode(sptr, UMODE_CHATOPS) && !IsULine(sptr)) {
		send_me_numericNA(sptr, ERR_NOPRIVILEGES);
		return 0;
	}
	if (parc < 2 || *parv[1] == '\0') {
		if (MyClient(sptr)) {
			send_me_numeric(sptr, ERR_NEEDMOREPARAMS, "CHATOPS");
		}
		return 0;
	}
	if (strlen(parv[1]) > TOPICLEN) {
		parv[1][TOPICLEN] = '\0';
	}
	sendto_serv_msg_butone(cptr, sptr, NULL, &CMD_CHATOPS, ":%s", parv[1]);
	send_chatops("from %s: %s", parv[0], parv[1]);
	return 0;
}
