/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_close.c: CLOSE command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_close.c,v 1.16 2004/04/09 01:45:52 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include "modules.h"
#include "xmode.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

Module MOD_HEADER(m_close) = {
	"modules/commands/m_close.so",
	"/CLOSE command",
	5, "$Revision: 1.16 $"
};

int MOD_LOAD(m_close)()
{
	register_command(&MOD_HEADER(m_close), &CMD_CLOSE, m_close);
	return MOD_SUCCESS;
}

int MOD_UNLOAD(m_close)()
{
	return MOD_SUCCESS;
}

/*
 * m_close
 *	parv[0] = sender prefix
 */
int m_close(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	dlink_node *node, *next = NULL;
	aClient *acptr;
	int closed = 0;

	if (!MyConnect(sptr) || !HasMode(sptr, UMODE_OPER)) {
		send_me_numericNA(sptr, ERR_NOPRIVILEGES);
		return 0;
	}

	DLINK_FOREACH_SAFE_DATA(lunknown_list.head, node, next, acptr, aClient) {
		send_me_numeric(sptr, RPL_CLOSING, get_client_name(acptr, TRUE), acptr->status);
		exit_client(acptr, acptr, &me, "Oper Closing");
		closed++;
	}

	send_me_numeric(sptr, RPL_CLOSEEND, closed);
	return 0;
}
