/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_connect.c: CONNECT command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_connect.c,v 1.33 2004/03/07 16:13:35 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include "modules.h"
#include "xmode.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>  
#include <string.h>
#include <fcntl.h>

Module MOD_HEADER(m_connect) = {
	"modules/commands/m_connect.so",
	"/CONNECT command",
	5, "$Revision: 1.33 $"
};

int MOD_LOAD(m_connect)()
{
	register_command(&MOD_HEADER(m_connect), &CMD_CONNECT, m_connect);
	return MOD_SUCCESS;
}

int MOD_UNLOAD(m_connect)()
{
	return MOD_SUCCESS;
}

/*
 * m_connect
 *	parv[0] = sender prefix
 *	parv[1] = servername
 *	parv[2] = port number
 *	parv[3] = remote server
 */
int m_connect(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	int port, tmpport;
	ConfigItem_link *linkp;
	aClient *acptr;
	char *inpath = NULL;

	if (!IsPrivileged(sptr) || (MyClient(sptr) && ((!OPHasFlag(sptr, OFLAG_LROUTE) && parc <= 3) ||
	  (!OPHasFlag(sptr, OFLAG_GROUTE) && parc > 3)))) {
		send_me_numericNA(sptr, ERR_NOPRIVILEGES);
		return 0;
	}
	if (parc < 2 || *parv[1] == '\0') {
		send_me_numeric(sptr, ERR_NEEDMOREPARAMS, "CONNECT");
		return 0;
	}
	if (use_or_deliver(cptr, sptr, &CMD_CONNECT, "%s %s :%s", 3, parc, parv) != HUNTED_ISME) {
		return 0;
	}
	if ((acptr = find_server(parv[1], NULL)) != NULL) {
		send_me_notice(sptr, ":*** Connect: server %s already exists from %s.",
			parv[1], acptr->from->name);
		return 0;
	}

	if ((linkp = find_link(parv[1], "*")) == NULL) {
		send_me_notice(sptr, ":*** Connect: no link configuration found for %s.", parv[1]);
		return 0;
	}

	tmpport = port = linkp->port;
	if (parc > 2 && !BadPtr(parv[2])) {
		if ((port = atoi(parv[2])) <= 0) {
			send_me_noticeNA(sptr, ":*** Connect: illegal port number.");
			return 0;
		}
	}

	sendto_gnotice("%s CONNECT %s %s from %s", MyClient(sptr) ? "Local" : "Remote",
		parv[1], (!BadPtr(parv[2])) ? parv[2] : "", parv[0]);
	sendto_serv_msg_butone(NULL, &me, NULL, &CMD_GNOTICE, ":%s CONNECT %s %s from %s",
		me.name, MyClient(sptr) ? "Local" : "Remote", parv[1],
		(!BadPtr(parv[2])) ? parv[2] : "", parv[0]);
	ircdlog(LOG_SERVER, "CONNECT from %s: %s %s", parv[0], parv[1],
		!BadPtr(parv[2]) ? parv[2] : "*");

	inpath = HasMode(sptr, UMODE_RSTAFF) ? linkp->host : "0.0.0.0";
	linkp->port = port;

	if (serv_connect(linkp, sptr)) {
		send_me_notice(sptr, ":*** Connecting to %s[%s].%d.", linkp->servername,
			inpath, linkp->port);
	}
	else {
		send_me_notice(sptr, ":*** Couldn't connect to %s[%s].%d.", linkp->servername,
			inpath, linkp->port);
	}

	linkp->port = tmpport;
	return 0;
}
