/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_info.c: INFO command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_info.c,v 1.25 2004/05/09 01:01:05 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include "modules.h"
#include "xmode.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

Module MOD_HEADER(m_info) = {
	"modules/commands/m_info.so",
	"/INFO command",
	6, "$Revision: 1.25 $"
};

int MOD_LOAD(m_info)()
{
	register_command(&MOD_HEADER(m_info), &CMD_INFO, m_info);
	return MOD_SUCCESS;
}

int MOD_UNLOAD(m_info)()
{
	return MOD_SUCCESS;
}

/*
 * m_info
 *	parv[0] = sender prefix
 *	parv[1] = servername
 */
int m_info(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	char **text;
	static time_t last_used = 0L;

	if (use_or_deliver(cptr, sptr, &CMD_INFO, ":%s", 1, parc, parv) != HUNTED_ISME) {
		return 0;
	}
	if (!HasMode(sptr, UMODE_OPER)) {
		if (FloodConfig.pace_wait_simple > 0 && (last_used + FloodConfig.pace_wait_simple > timeofday)) {
			send_me_numericNA(sptr, RPL_LOAD2HI);
			return 0;
		}
		else {
			last_used = timeofday;
		}
		if (GeneralConfig.spy_notices && IsPerson(sptr)) {
			sendto_realops_lev(SPY_LEV, "INFO requested by %s (%s@%s) [%s]", sptr->name,
				sptr->username, MaskedHost(sptr), sptr->user->server);
		}
	}

	for (text = infotext; *text != NULL; text++) {
		send_me_numeric(sptr, RPL_INFO, *text);
	}

	send_me_numeric_buf(sptr, ":Birth date: %s, compile # %s", RPL_INFO, creation, generation);
	send_me_numeric_buf(sptr, ":On-line since: %s", RPL_INFO, myctime(me.firsttime));

	send_me_numericNA(sptr, RPL_ENDOFINFO);
	return 0;
}
