/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_pass.c: PASS protocol module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_pass.c,v 1.17 2004/04/08 20:54:23 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include "modules.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

Module MOD_HEADER(m_pass) = {
	"modules/commands/m_pass.so",
	"PASS protocol",
	5, "$Revision: 1.17 $"
};

int MOD_LOAD(m_pass)()
{
	register_command(&MOD_HEADER(m_pass), &CMD_PASS, m_pass);
	return MOD_SUCCESS;
}

int MOD_UNLOAD(m_pass)()
{
	return MOD_SUCCESS;
}

/*
 * m_pass
 *	parv[0] = sender prefix
 *	parv[1] = password
 *	parv[2] = optional version information
 */
int m_pass(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	char *password = parc > 1 ? parv[1] : NULL;

	if (BadPtr(password)) {
		send_me_numeric(sptr, ERR_NEEDMOREPARAMS, "PASS");
		return 0;
	}
	if (!MyConnect(sptr) || (!IsUnknown(cptr) && !IsHandshake(cptr))) {
		send_me_numericNA(cptr, ERR_ALREADYREGISTRED);
		return 0;
	}

	ASSERT(cptr->localClient != NULL);

	strncpyzt(cptr->localClient->passwd, password, PASSWDLEN + 1);
	if (parc > 2 && strlen(parv[2]) > 1) {
		if (*parv[2] == 'T' && *(parv[2] + 1) == 'S') {
			cptr->localClient->capabs |= CAP_TS;
		}
	}
	return 0;
}
