
/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_ping.c: PING command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_ping.c,v 1.18 2004/05/08 16:12:02 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include "modules.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

Module MOD_HEADER(m_ping) = {
	"modules/commands/m_ping.so",
	"/PING command",
	6, "$Revision: 1.18 $"
};

int MOD_LOAD(m_ping)()
{
	register_command(&MOD_HEADER(m_ping), &CMD_PING, m_ping);
	MOD_SET_FLAG(&MOD_HEADER(m_ping), MOD_FLAG_PERM);
	return MOD_SUCCESS;
}

int MOD_UNLOAD(m_ping)()
{
	return MOD_SUCCESS;
}

/*
 * m_ping
 *	parv[0] = sender prefix
 *	parv[1] = origin
 *	parv[2] = destination
 */
int m_ping(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	aClient *acptr;
	char *origin, *destination;

	if (parc < 2 || *parv[1] == '\0') {
		send_me_numericNA(sptr, ERR_NOORIGIN);
		return 0;
	}

	origin = parv[1];
	destination = parv[2];

	if ((acptr = find_client(origin, NULL)) == NULL) {
		acptr = find_server(origin, NULL);
	}
	if (acptr != NULL && acptr != sptr) {
		origin = cptr->name;
	}
	if (!BadPtr(destination) && mycmp(destination, me.name)) {
		if ((acptr = find_server(destination, NULL)) == NULL) {
			send_me_numeric(sptr, ERR_NOSUCHSERVER, destination);
		}
		else {
			sendto_one_client_nopostfix(acptr, sptr, &CMD_PING, "%s :%s", origin,
				destination);
		}
		return 0;
	}
	sendto_one_client_nopostfix(sptr, &me, &CMD_PONG, "%s :%s",
		destination ? destination : me.name, origin);
	return 0;
}
