/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_sajoin.c: SAJOIN command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_sajoin.c,v 1.28 2004/05/13 16:12:54 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include "hook.h"
#include "modules.h"
#include "xmode.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

extern chanMember *add_user_to_channel(aChannel *, aClient *, int);

Hook *hp_post_join;

Module MOD_HEADER(m_sajoin) = {
	"modules/commands/m_sajoin.so",
	"/SAJOIN command",
	6, "$Revision: 1.28 $"
};

int MOD_LOAD(m_sajoin)()
{
	if ((hp_post_join = hook_find_from_module("h_post_join", "modules/commands/m_join.so")) == NULL) {
		ircdlog(LOG_ERROR, "m_sajoin.so: couldn't find hook h_post_join");
		return MOD_FAILURE;
	}

	register_command(&MOD_HEADER(m_sajoin), &CMD_SAJOIN, m_sajoin);
	return MOD_SUCCESS;
}

int MOD_UNLOAD(m_sajoin)()
{
	return MOD_SUCCESS;
}

/*
 * m_sajoin
 *	parv[0] = sender prefix
 *	parv[1] = channel
 */
int m_sajoin(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	aChannel *chptr;
	char *p;

	if (!IsPrivileged(sptr) || !HasMode(sptr, UMODE_SADMIN)) {
		send_me_numericNA(sptr, ERR_NOPRIVILEGES);
		return 0;
	}
	if (parc < 2) {
		send_me_numeric(sptr, ERR_NEEDMOREPARAMS, "SAJOIN");
		return 0;
	}
	if ((p = strchr(parv[1], ',')) != NULL) {
		*p = '\0';
	}
	if ((chptr = find_channel(parv[1], NULL)) == NULL) {
		if (!MyClient(sptr)) {
			sendto_realops_lev(DEBUG_LEV, "DESYNCH: client %s!%s@%s has SAJOIN'ed nonexistant "
				"channel %s", sptr->name, sptr->username, MaskedHost(sptr), parv[1]);
			ircdlog(LOG_ERROR, "DESYNCH: client %s has SAJOIN'ed nonexistant channel %s",
				get_client_name(sptr, FALSE), parv[1]);
		}
		else {
			send_me_numeric(sptr, ERR_NOSUCHCHANNEL, parv[1]);
		}
		return 0;
	}
	if (IsMember(sptr, chptr)) {
		return 0;
	}

	sendto_realops("%s used SAJOIN %s", sptr->name, chptr->chname);
	sendto_serv_msg_butone(NULL, &me, NULL, &CMD_GLOBOPS, ":%s used SAJOIN %s", sptr->name,
		chptr->chname);
	ircdlog(LOG_OVERRIDE, "%s used SAJOIN %s", get_client_name(sptr, FALSE), chptr->chname);

	add_user_to_channel(chptr, sptr, 0);
	sendto_serv_msg_butone(cptr, sptr, NULL, &CMD_SJOIN, "%ld %s", chptr->channelts, chptr->chname);
	send_channel_msg_local_copy(sptr, chptr, &CMD_JOIN, PREFIX_NONE, "");

	if (MyClient(sptr)) {
		HookData hdata;

		if (*chptr->topic != '\0') {
			send_me_numeric(sptr, RPL_TOPIC, chptr->chname, chptr->topic);
			send_me_numeric(sptr, RPL_TOPICWHOTIME, chptr->chname, chptr->topic_nick,
				chptr->topic_time);
		}

		hdata.cptr = cptr;
		hdata.sptr = sptr;
		hdata.chptr = chptr;
		hook_run(hp_post_join, &hdata);
	}
	return 0;
}
