/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_sgline.c: SGLINE command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_sgline.c,v 1.19 2004/03/07 16:58:30 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include "modules.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

Module MOD_HEADER(m_sgline) = {
	"modules/commands/m_sgline.so",
	"/SGLINE command",
	5, "$Revision: 1.19 $"
};

int MOD_LOAD(m_sgline)()
{
	register_command(&MOD_HEADER(m_sgline), &CMD_SGLINE, m_sgline);
	return MOD_SUCCESS;
}

int MOD_UNLOAD(m_sgline)()
{
	return MOD_SUCCESS;
}

int m_sgline(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	ConfigItem_ban *ban;
	int len;
	char *mask, *reason;

	if (!(IsServer(sptr) || IsULine(sptr))) {
		return 0;
	}
	if (parc < 3) {
		send_me_numeric(sptr, ERR_NEEDMOREPARAMS, "SGLINE");
		return 0;
	}

	len = atoi(parv[1]);
	mask = parv[2];
	if (strlen(mask) > len && mask[len] == ':') {
		mask[len] = '\0';
		reason = mask + len + 1;
	}
	else {
		return 0;
	}
	if ((ban = find_ban(BAN_GCOS, mask)) == NULL) {
		ban = (ConfigItem_ban *)MyMallocX(sizeof(ConfigItem_ban));
		ban->item.temp = 0;
		ban->flags |= (BAN_GCOS|BAN_SVS);
		DupString(ban->mask, mask);
		DupString(ban->reason, !BadPtr(reason) ? reason : "no reason");
		dlink_add(&conf_ban_list, ban);
	}
	sendto_serv_msg_butone(cptr, sptr, NULL, &CMD_SGLINE, "%d :%s:%s", len, ban->mask, ban->reason);
	return 0;
}
