/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_svsmode.c: SVSMODE command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_svsmode.c,v 1.21 2004/04/06 22:10:32 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include "modules.h"
#include "xmode.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

Module MOD_HEADER(m_svsmode) = {
	"modules/commands/m_svsmode.so",
	"/SVSMODE command",
	5, "$Revision: 1.21 $"
};

int MOD_LOAD(m_svsmode)()
{
	register_command(&MOD_HEADER(m_svsmode), &CMD_SVSMODE, m_svsmode);
	return MOD_SUCCESS;
}

int MOD_UNLOAD(m_svsmode)()
{
	return MOD_SUCCESS;
}

/*
 * m_svsmode
 *	parv[0] = sender prefix
 *	parv[1] = nickname
 *	parv[2] = TS (or mode, depending on SVS version)
 *	parv[3] = mode (or serviceid, if old SVS version)
 *	parv[4] = optional argument (serviceid)
 */
int m_svsmode(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	aClient *acptr;
	char *modes, *opt_arg = NULL, *p;
	xMode *t = NULL;
	short change = XMODE_ADD;
	unsigned long old_umodes = 0;
	time_t ts = 0;
	int i;

	if (!IsULine(sptr) || parc < 3) {
		return 0;
	}
	if (parc >= 4 && (*parv[3] == '+' || *parv[3] == '-')) {
		ts = atol(parv[2]);
		modes = parv[3];
		opt_arg = (parc > 4) ? parv[4] : NULL;
	}
	else {
		modes = parv[2];
		opt_arg = (parc > 3) ? parv[3] : NULL;
	}

	if ((acptr = find_person_target(parv[1])) == NULL) {
		return 0;
	}
	if (ts && (ts != acptr->tsinfo)) {
		return 0;
	}

	old_umodes = acptr->umode;

	for (p = modes; *p != '\0'; p++) {
		switch (*p) {
			case '+':
				change = XMODE_ADD;
				break;
			case '-':
				change = XMODE_DEL;
				break;
			case ' ':
			case '\n':
			case '\r':
			case '\t':
				break;
			case 'd':
				if (opt_arg != NULL && isdigit(*opt_arg)) {
					acptr->user->servicestamp = strtoul(opt_arg, NULL, 0);
				}
				break;
			default:
				if ((i = usermodes->map[(unsigned char)*p]) != -1) {
					t = &usermodes->table[i];

					if (change == XMODE_ADD) {
						acptr->umode |= t->mode;
					}
					else {
						acptr->umode &= ~t->mode;
					}
					if (MyConnect(acptr) && (change == XMODE_DEL)
					  && (t->mode == UMODE_OPER) && !HasMode(acptr, UMODE_OPER)) {
						dlink_del(&oper_list, acptr, NULL);
					}
				}
				break;
		}
	}
	if (opt_arg != NULL) {
		sendto_serv_msg_butone(cptr, sptr, acptr, &CMD_SVSMODE, "%ld %s %s", acptr->tsinfo,
			modes, opt_arg);
	}
	else {
		sendto_serv_msg_butone(cptr, sptr, acptr, &CMD_SVSMODE, "%ld %s", acptr->tsinfo, modes);
	}

	if (MyClient(acptr) && (old_umodes != acptr->umode)) {
		char buf[BUFSIZE];
		send_umode(acptr, acptr, old_umodes, ALL_UMODES, buf);
	}
	return 0;
}
