/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_svsnoop.c: SVSNOOP command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_svsnoop.c,v 1.22 2004/03/07 16:13:35 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include "modules.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

Module MOD_HEADER(m_svsnoop) = {
	"modules/commands/m_svsnoop.so",
	"/SVSNOOP command",
	5, "$Revision: 1.22 $"
};

int MOD_LOAD(m_svsnoop)()
{
	register_command(&MOD_HEADER(m_svsnoop), &CMD_SVSNOOP, m_svsnoop);
	return MOD_SUCCESS;
}

int MOD_UNLOAD(m_svsnoop)()
{
	return MOD_SUCCESS;
}

/*
 * m_svsnoop
 *	parv[0] = sender prefix
 *	parv[1] = server
 *	parv[2] = +/-
 */
int m_svsnoop(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	dlink_node *node, *next = NULL;
	ConfigItem_oper *oper = NULL;

	if (!IsULine(sptr) && parc < 2) {
		return 0;
	}
	if (use_or_deliver(cptr, sptr, &CMD_SVSNOOP, "%s :%s", 1, parc, parv) != HUNTED_ISME) {
		return 0;
	}

	if (*parv[2] != '+') {
		conf_rehash();
		return 0;
	}
	DLINK_FOREACH_SAFE_DATA(conf_oper_list.head, node, next, oper, ConfigItem_oper) {
		dlink_del(&conf_oper_list, NULL, node);
		free_oper(oper);
	}
	return 0;
}
