/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_unszline.c: UNSZLINE command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_unszline.c,v 1.11 2004/03/05 19:27:33 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include "modules.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

Module MOD_HEADER(m_unszline) = {
	"modules/commands/m_unszline.so",
	"/UNSZLINE command",
	5, "$Revision: 1.11 $"
};

int MOD_LOAD(m_unszline)()
{
	register_command(&MOD_HEADER(m_unszline), &CMD_UNSZLINE, m_unszline);
	return MOD_SUCCESS;
}

int MOD_UNLOAD(m_unszline)()
{
	return MOD_SUCCESS;
}

/*
 * m_unszline
 *	parv[0] = sender prefix
 *	parv[1] = nickname
 */
int m_unszline(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	dlink_node *node, *next = NULL;
	ConfigItem_ban *ban;
	int matchit = 0;
	char *mask;

	if (!(IsServer(sptr) || IsULine(sptr))) {
		return 0;
	}
	if (parc < 2) {
		send_me_numeric(sptr, ERR_NEEDMOREPARAMS, "UNSQLINE");
		return 0;
	}
	if (parc == 3) {
		matchit = atoi(parv[1]);
		mask = parv[2];
	}
	else {
		mask = parv[1];
	}

	DLINK_FOREACH_SAFE_DATA(conf_ban_list.head, node, next, ban, ConfigItem_ban) {
		if ((ban->flags & (BAN_IP|BAN_SVS)) != (BAN_IP|BAN_SVS)) {
			continue;
		}
		if (matchit ? !match(mask, ban->mask) : !mycmp(mask, ban->mask)) {
			dlink_del(&conf_ban_list, NULL, node);
			free_ban(ban);
		}
	}

	if (parc == 3) {
		sendto_serv_msg_butone(cptr, sptr, NULL, &CMD_UNSZLINE, "%d :%s", matchit, mask);
	}
	else {
		sendto_serv_msg_butone(cptr, sptr, NULL, &CMD_UNSZLINE, ":%s", mask);
	}
	return 0;
}
