/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_userhost.c: USERHOST command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_userhost.c,v 1.18 2004/04/01 23:47:56 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include "modules.h"
#include "xmode.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

Module MOD_HEADER(m_userhost) = {
	"modules/commands/m_userhost.so",
	"/USERHOST command",
	5, "$Revision: 1.18 $"
};

int MOD_LOAD(m_userhost)()
{
	register_command(&MOD_HEADER(m_userhost), &CMD_USERHOST, m_userhost);
	return MOD_SUCCESS;
}

int MOD_UNLOAD(m_userhost)()
{
	return MOD_SUCCESS;
}

/*
 * m_userhost
 *	parv[0] = sender prefix
 *	parv[1] = user
 */
int m_userhost(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	static char buf[BUFSIZE];
	char buf2[NICKLEN + USERLEN + HOSTLEN + 5];
	char *s, *p = NULL;
	aClient *acptr;
	int idx = 0, len, mlen, i = 5;

	if (parc < 2) {
		send_me_numeric(sptr, ERR_NEEDMOREPARAMS, "USERHOST");
		return 0;
	}

	mlen = strlen(me.name) + strlen(sptr->name) + 12;
	for (s = strtoken(&p, parv[1], " "); (i > 0) && (s != NULL); s = strtoken(&p, NULL, " ")) {
		i--;

		if ((acptr = find_person(s, NULL)) == NULL) {
			continue;
		}

		len = ircsprintf(buf2, "%s%s=%c%s@%s", acptr->name, HasMode(sptr, UMODE_OPER) ? "*" : "",
			BadPtr(acptr->user->away) ? '+' : '-', acptr->username, MaskedHost(acptr));

		if ((mlen + idx + len) > BUFSIZE - 1) {
			break;
		}
		ADD_STRING(buf2, buf, idx);
	}

	send_me_numeric(sptr, RPL_USERHOST, buf);
	return 0;
}
