/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_wallops.c: WALLOPS command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_wallops.c,v 1.15 2004/03/05 19:27:33 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include "modules.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

Module MOD_HEADER(m_wallops) = {
	"modules/commands/m_wallops.so",
	"/WALLOPS command",
	5, "$Revision: 1.15 $"
};

int MOD_LOAD(m_wallops)()
{
	register_command(&MOD_HEADER(m_wallops), &CMD_WALLOPS, m_wallops);
	return MOD_SUCCESS;
}

int MOD_UNLOAD(m_wallops)()
{
	return MOD_SUCCESS;
}

/*
 * m_wallops
 *	parv[0] = sender prefix
 *	parv[1] = message
 */
int m_wallops(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	if (!IsServer(sptr) && MyConnect(sptr) && !OPHasFlag(sptr, OFLAG_WALLOPS)) {
		send_me_numericNA(sptr, ERR_NOPRIVILEGES);
		return 0;
	}
	if (parc < 2 || *parv[1] == '\0') {
		send_me_numeric(sptr, ERR_NEEDMOREPARAMS, "WALLOPS");
		return 0;
	}
	sendto_wallops(sptr, ":%s WALLOPS :%s", parv[0], parv[1]);
	sendto_serv_msg_butone(cptr, sptr, NULL, &CMD_WALLOPS, ":%s", parv[1]);
	return 0;
}
