/*
 * RageIRCd, an advanced Internet Relay Chat daemon (IRCd)
 * modules/m_whowas.c: WHOWAS command module
 *
 * Copyright (C) 2002 the past and present IRCd developers, and others.
 * Refer to doc/authors/ for full credits and copyright details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * $Id: m_whowas.c,v 1.24 2004/05/13 17:43:51 rageircd Exp $
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"
#include "memory.h"
#include "modules.h"
#include "xmode.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

extern char *canonize(char *);
extern void init_whowas();
extern unsigned int hash_whowas_name(char *);
extern aWhowas WHOWAS[NICKNAMEHISTORYLENGTH];
extern aWhowas *WHOWASHASH[WW_MAX];

Module MOD_HEADER(m_whowas) = {
	"modules/commands/m_whowas.so",
	"/WHOWAS command",
	6, "$Revision: 1.24 $"
};

int MOD_LOAD(m_whowas)()
{
	init_whowas();
	register_command(&MOD_HEADER(m_whowas), &CMD_WHOWAS, m_whowas);
	return MOD_SUCCESS;
}

int MOD_UNLOAD(m_whowas)()
{
	return MOD_SUCCESS;
}

/*
 * m_whowas
 *	parv[0] = sender prefix
 *	parv[1] = nickname queried
 */
int m_whowas(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	aWhowas *tmp = NULL;
	int cur = 0, found = 0, max = -1;
	char *p, *nick, *s;
	static time_t last_used = 0L;

	if (parc < 2) {
		send_me_numeric(sptr, ERR_NEEDMOREPARAMS, "WHOWAS");
		return 0;
	}
	if (parc > 2) {
		max = atoi(parv[2]);
	}

	if (!HasMode(sptr, UMODE_OPER)) {
		if (FloodConfig.pace_wait_intense > 0 && (last_used + FloodConfig.pace_wait_intense > timeofday)) {
			send_me_numericNA(sptr, RPL_LOAD2HI);
			return 0;
		}
		else {
			last_used = timeofday;
		}
	}
	if (parc > 3) {
		if (use_or_deliver(cptr, sptr, &CMD_WHOWAS, "%s %s :%s", 3, parc, parv)) {
			return 0;
		}
	}

	parv[1] = canonize(parv[1]);
	if (!MyConnect(sptr) && max > 20) {
		max = 20;
	}
	for (s = parv[1]; (nick = strtoken(&p, s, ",")) != NULL; s = NULL) {
		found = 0;
		for (tmp = WHOWASHASH[hash_whowas_name(nick)]; tmp != NULL; tmp = tmp->next) {
			if (!mycmp(nick, tmp->name)) {
				send_me_numeric(sptr, RPL_WHOWASUSER, tmp->name, tmp->username,
					tmp->maskedhost, tmp->realname);
				if (HasMode(sptr, UMODE_OPER)) {
					send_me_numeric(sptr, RPL_WHOWASHOST, tmp->name,
						tmp->hostname);
				}
				send_me_numeric(sptr, RPL_WHOISSERVER, tmp->name, tmp->servername,
					myctime(tmp->logoff));
				cur++;
				found++;
			}
			if (max > 0 && cur > max) {
				break;
			}
		}
		if (!found) {
			send_me_numeric(sptr, ERR_WASNOSUCHNICK, nick);
		}
		if (p != NULL) {
			p[-1] = ',';
		}
	}
	send_me_numeric(sptr, RPL_ENDOFWHOWAS, parv[1]);
	return 0;
}
